@@ -1,5 +1,3 @@
-// $Id: BPF_Program.cc 6219 2008-10-01 05:39:07Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
@@ -60,9 +58,8 @@ int pcap_compile_nopcap(int snaplen_arg, int linktype_arg,
 }
 #endif
 
-BPF_Program::BPF_Program()
+BPF_Program::BPF_Program() : m_compiled(), m_program()
 	{
-	m_compiled = false;
 	}
 
 BPF_Program::~BPF_Program()
