@@ -1,5 +1,3 @@
-// $Id: EventRegistry.h 6829 2009-07-09 09:12:59Z vern $
-//
 // Each event raised/handled by Bro is registered in the EventRegistry.
 
 #ifndef EVENT_REGISTRY
@@ -28,11 +26,10 @@ public:
 	typedef PList(constchar) string_list;
 	string_list* Match(RE_Matcher* pattern);
 
-	// Associates a group with the given event.
-	void SetGroup(const char* name, const char* group);
-
-	// Enable/disable all members of the group.
-	void EnableGroup(const char* group, bool enable);
+	// Marks a handler as handling errors. Error handler will not be called
+	// recursively to avoid infinite loops in case they trigger an error
+	// themselves.
+	void SetErrorHandler(const char* name);
 
 	string_list* UnusedHandlers();
 	string_list* UsedHandlers();
